// Mux1 control pins
const short M1S0 = 0;
const short M1S1 = 1;
const short M1S2 = 2;
const short M1S3 = 3;

const short M1SIG = 16;

const short M1PINS[4] = {M1S0, M1S1, M1S2, M1S3};

// Mux2 control pins
const short M2S0 = 4;
const short M2S1 = 5;
const short M2S2 = 6;
const short M2S3 = 7;

const short M2SIG = 17;

const short M2PINS[4] = {M2S0, M2S1, M2S2, M2S3};

// Sensor to Midi Note / Mux1
const short notes[30] = {
  69, 57, 45, 33, 21,
  71, 59, 47, 35, 23,
  72, 60, 48, 36, 24,
  74, 62, 50, 38, 26,
  76, 64, 52, 40, 28,
  77, 65, 53, 41, 29
};

const int muxChannel[16][4] = {
    {0,0,0,0}, //channel 0
    {1,0,0,0}, //channel 1
    {0,1,0,0}, //channel 2
    {1,1,0,0}, //channel 3
    {0,0,1,0}, //channel 4
    {1,0,1,0}, //channel 5
    {0,1,1,0}, //channel 6
    {1,1,1,0}, //channel 7
    {0,0,0,1}, //channel 8
    {1,0,0,1}, //channel 9
    {0,1,0,1}, //channel 10
    {1,1,0,1}, //channel 11
    {0,0,1,1}, //channel 12
    {1,0,1,1}, //channel 13
    {0,1,1,1}, //channel 14
    {1,1,1,1}  //channel 15
};

/*  Start values
 *  0 = current value from analog in
 *  1 = differenz between current and next value
 *  2 = next value from analog in
 */
float values[30][4];

// Threshold when to send usb midi
float threshold[30] = {
      3.0, 3.0, 3.0, 3.0, 3.0,
      3.0, 3.0, 3.0, 3.0, 3.0,
      3.0, 3.0, 3.0, 3.0, 3.0,
      3.0, 3.0, 3.0, 3.0, 3.0,
      3.0, 3.0, 3.0, 3.0, 3.0,
      3.0, 3.0, 3.0, 3.0, 3.0
};

// Used cycle through sample for
float tval = 0, trow = 0;

// short sample isze
const short sampleSize = 66;

void setup() {

  for(int i = 0; i < 30; i++) {
    values[i][0] = 0;
    values[i][1] = 0;
    values[i][2] = 0;
    values[i][3] = 0;
  }
  
  // Initializing Muxes 
  // M1
  pinMode(M1S0, OUTPUT);
  pinMode(M1S1, OUTPUT);
  pinMode(M1S2, OUTPUT);
  pinMode(M1S3, OUTPUT); 
  // M2
  pinMode(M2S0, OUTPUT);
  pinMode(M2S1, OUTPUT);
  pinMode(M2S2, OUTPUT);
  pinMode(M2S3, OUTPUT);   

  // M1
  digitalWrite(M1S0, LOW);
  digitalWrite(M1S1, LOW);
  digitalWrite(M1S2, LOW);
  digitalWrite(M1S3, LOW);

  // M2
  digitalWrite(M2S0, LOW);
  digitalWrite(M2S1, LOW);
  digitalWrite(M2S2, LOW);
  digitalWrite(M2S3, LOW);

  Serial.begin(9600);
}

void loop() {
  // READ M1
  for(short p = 0; p < 15; p++) {
    tval = 0;
    for(int i = 0; i < sampleSize; i++) {
      tval += readMux(p, M1PINS, M1SIG);
    }
    
    tval /= sampleSize;

    if(values[p][0] == 0) {
      values[p][0] = tval;
      values[p][1] = 0;
      values[p][2] = tval;
    } else {
      values[p][0] = values[p][2];
      values[p][1] = roundK(tval-values[p][0]);
      values[p][2] = tval;
    }
  }
  
  // READ M2
  for(short p = 0; p < 15; p++) {
    tval = 0;
    for(int i = 0; i < sampleSize; i++) {
      tval += readMux(p, M2PINS, M2SIG);
    }
    
    tval /= sampleSize;

    if(values[p+15][0] == 0) {
      values[p+15][0] = tval;
      values[p+15][1] = 0;
      values[p+15][2] = tval;
    } else {
      values[p+15][0] = values[p+15][2];
      values[p+15][1] = roundK(tval-values[p+15][0]);
      values[p+15][2] = tval;
    }
  }  
  
  /* Console output */
  for(short p = 0; p < 30; p++) {
      if(p < 15) {
        Serial.print("M1 ");
        if(p < 10) Serial.print("0");
        Serial.print(p);
      } else {
        Serial.print("M2 ");
        if(p-15 < 10) Serial.print("0");
        Serial.print(p-15);       
      }
      
      Serial.print(" : ");
      
      Serial.print(values[p][1]);
      
      Serial.print(" | ");
      
      Serial.print(threshold[p]);
      
      Serial.println();
   }
   
   /* Check & play single sounds */
  for(short p = 0; p < 30; p++) {
    if(abs(values[p][1]) > threshold[p]) usbMIDI.sendNoteOn(notes[p], 127, 1);
    else usbMIDI.sendNoteOff(notes[p], 0, 1);
  }
  
  /* Check and play base noise */
  // Row 1
  trow = abs(values[0][1]) + abs(values[1][1]) + abs(values[2][1]) + abs(values[3][1]) + abs(values[4][1]);
  Serial.print("Row 1 sum: ");
  Serial.print(trow);
  Serial.println();
  if(trow > 8.0) usbMIDI.sendNoteOn(9, 127, 2);
  else usbMIDI.sendNoteOff(9, 0, 2);
  // Row 2
  trow = abs(values[5][1]) + abs(values[6][1]) + abs(values[7][1]) + abs(values[8][1]) + abs(values[9][1]);
  Serial.print("Row 2 sum: ");
  Serial.print(trow);
  Serial.println();
  if(trow > 8.0) usbMIDI.sendNoteOn(11, 127, 2);
  else usbMIDI.sendNoteOff(11, 0, 2);   
  // Row 3
  trow = abs(values[10][1]) + abs(values[11][1]) + abs(values[12][1]) + abs(values[13][1]) + abs(values[14][1]);
  Serial.print("Row 3 sum: ");
  Serial.print(trow);
  Serial.println();
  if(trow > 8.0) usbMIDI.sendNoteOn(12, 127, 2);
  else usbMIDI.sendNoteOff(12, 0, 2);
  // Row 4
  trow = abs(values[15][1]) + abs(values[16][1]) + abs(values[17][1]) + abs(values[18][1]) + abs(values[19][1]);
  Serial.print("Row 4 sum: ");
  Serial.print(trow);
  Serial.println();
  if(trow > 10.0) usbMIDI.sendNoteOn(14, 127, 2);
  else usbMIDI.sendNoteOff(14, 0, 2);  
  // Row 5
  trow = abs(values[20][1]) + abs(values[21][1]) + abs(values[22][1]) + abs(values[23][1]) + abs(values[24][1]);
  Serial.print("Row 5 sum: ");
  Serial.print(trow);
  Serial.println();
  if(trow > 18.0) usbMIDI.sendNoteOn(16, 127, 2);
  else usbMIDI.sendNoteOff(16, 0, 2);    
  // Row 6
  trow = abs(values[25][1]) + abs(values[26][1]) + abs(values[27][1]) + abs(values[28][1]) + abs(values[29][1]);
  Serial.print("Row 6 sum: ");
  Serial.print(trow);
  Serial.println();
  if(trow > 16.0) usbMIDI.sendNoteOn(17, 127, 2);
  else usbMIDI.sendNoteOff(17, 0, 2);      
}

int readMux(short channel, const short pins[], short sigPin) {
  //loop through the 4 sig
  digitalWrite(pins[0], muxChannel[channel][0]);
  digitalWrite(pins[1], muxChannel[channel][1]);
  digitalWrite(pins[2], muxChannel[channel][2]);
  digitalWrite(pins[3], muxChannel[channel][3]);

  //read the value at the SIG pin
  return analogRead(sigPin);
}

int roundK(float v) {
  return ((int)(v+.5));
}
